+{
   locale_version => 1.27,
   entry => <<'ENTRY', # for DUCET v10.0.0
04D9      ; [.20C1.0020.0002] # CYRILLIC SMALL LETTER SCHWA
04D8      ; [.20C1.0020.0008] # CYRILLIC CAPITAL LETTER SCHWA
ENTRY
};
